### A Pluto.jl notebook ###
# v0.19.20

using Markdown
using InteractiveUtils

# ╔═╡ 5ba10044-c01a-486a-b6bd-bad198bf1960
# Braket imports
using Braket, PyBraket, SparseArrays, NLopt

# ╔═╡ ec25630f-2ca7-421a-8559-3046144dd236
using Graphs, GraphPlot, Colors, Cairo, Compose, LinearAlgebra

# ╔═╡ 7112d37e-9f1e-11ed-313d-11843dc0a14e
md"""
### Introduction to QAOA and Graph Coloring
"""

# ╔═╡ 12feb970-e7e0-4e96-b537-5afe8e82ac17
md"""
In this notebook, we'll introduce the problem of graph coloring and the quantum approximate optimization algorithm (QAOA). We'll write the graph coloring problem in a form that is amenable to running on a quantum device and use a simulator to run some small examples.
"""

# ╔═╡ 318db32f-e76c-4a46-9557-bd93a660505a
md"""
## Graph coloring: what and why?

Many optimization problems in the real world can be modeled in terms of "graph coloring". In such a problem, we have a set of vertices connected by edges (a graph) and would like to place each vertex in one of a set of bins such that no adjacent vertices are in the same bin. You might have heard of this problem in the context of coloring a map - we don't want neighboring countries on a map to have the same color or it will be hard to tell them apart. But there are other problems that are naturally expressible as a graph coloring problem, such as:
- Scheduling tasks to run on GPU workers
- Creating exam schedules for university students
- Assigning gates to aircraft at the airport
- In general, any scheduling problem in which many clients/workers (students, CPU threads, airplanes) contend for scarce resources (exam slots, exclusive control over a GPU, aircraft gates) and conflicts may occur (simultaneous exams, threads getting blocked, planes running into each other)
"""

# ╔═╡ 3023c427-e656-4468-951e-669952b3124b
md"""
## Introduction to QAOA

There are many existing algorithms developed to solve graph coloring problems. Why would we want another?

The quantum approximate optimization algorithm (QAOA) was developed to make use of near term quantum devices. Quantum computers today don't have a large number of qubits, but we would still like to use them to solve real-world problems. QAOA is based on the concept of *hybrid computations* -- accelerating a classical workload using a quantum processor. Similar to computing using a GPU or TPU, we hope to offload the hard part of the computation to a quantum device which acts as an accelerator.

QAOA is designed to solve "QUBO" problems: quadratic unconstrained binary optimization. A QUBO problem is one that can be formulated as:

Find the vector $\vec{y}$ which minimizes $\langle\vec{z} , Q \vec{z}\rangle$.

`Q` is a matrix which encodes the optimization problem/cost function. To understand the basics of QAOA, let's start with a quantum formulation of the QUBO form above by replacing `z^T Q z` with a Hamiltonian `\hat{H}` such that:

$\hat{H} = \sum_{i,j} J_{i,j} \hat{\sigma}^z_i \hat{\sigma}^z_j$

The sum runs over all pairs of sites in the system. This is an Ising model of magnetism of quantum spins, and we can use a quantum computer to solve it. If we find the ground state of this quantum system we have solved the associated QUBO problem. The ground state is the eigenvector of the matrix `H` associated with the lowest-lying eigenvalue. QAOA attempts to do this by applying a series of circuits, each of which takes a variational parameter, and optimizing the parameters.
"""

# ╔═╡ b71a0897-b1e3-4ea7-89e0-53ff4aca5e57
md"""
### Writing the building blocks of QAOA

In QAOA we apply two sub-circuits in each layer: the "cost" circuit and the "driver" circuit. The cost circuit has parameter γ associated with it, and the driver has parameter β. These parameters are optimized as the algorithm runs to arrive at a good solution to the optimization problem. The classical computer handles the parameter updates and the quantum device handles the preparation and computation of $\langle \psi | \hat{H} | \psi \rangle$. We'll first build a generic interface to create the cost and driver circuits:
"""

# ╔═╡ a1592cdc-0f2e-4794-96f7-328347b27481
"""
	ZZgate(q1, q2, γ) -> Circuit

Implementation of `ZZ` gate with native gates for QPUs (Rigetti) that do not
support the `ZZ` gate.
"""
function ZZgate(q1, q2, γ)
    circ_zz = Circuit([(CNot, q1, q2), (Rz, q2, γ), (CNot, q1, q2)])
    return circ_zz
end

# ╔═╡ c32edfaf-5191-4428-b86f-4077222b1b06
"""
	cost_circuit(γ, nqubits::Int, device::Device, ising::SparseMatrixCSC) -> Circuit

Builds the cost portion of the QAOA circuit. If the device supports the `ZZ` gate natively, that is used, otherwise use a compiled version.
"""
function cost_circuit(γ, nqubits::Int, device::Device, ising::SparseMatrixCSC)
    cost    = Circuit()
    I, J, V = findnz(ising)
    for (i, j, v) in zip(I, J, V)
        if i != j
            if name(device) == "Rigetti"
                append!(cost, ZZgate(i-1, j-1, γ*v))
            else
                cost(ZZ, i-1, j-1, -2γ*v)
            end
        end
    end
    return cost
end

# ╔═╡ 03970aa4-a5e1-44cf-b06f-dfabe28913d8
"""
	driver_circuit(β, nqubits::Int) -> Circuit

Builds the driver portion of the QAOA circuit.
"""
function driver_circuit(β, nqubits::Int)
    driver = Circuit()
    for qubit in 0:nqubits-11
        driver(Rx, qubit, 2β)
    end
    return driver
end

# ╔═╡ fd229791-a4a5-4b59-8ca5-e694a25d698d
"""
	qaoa_circuit(params::AbstractVector, device::Device, nqubits::Int, ising::SparseMatrixCSC) -> Circuit

Builds the full QAOA circuit using `params` applied to `cost_circuit` and `driver_circuit`. The number of layers is inferred from the size of `params`.
"""
function qaoa_circuit(params::AbstractVector, device::Device, nqubits::Int, ising::SparseMatrixCSC)
    depth = div(length(params), 2)
    γs    = params[depth+1:end]
    βs    = params[1:depth]

    circ = Circuit([(X, collect(0:nqubits-1)), (H, collect(0:nqubits-1))])

    for layer in 1:depth
        cost_circ = cost_circuit(γs[layer], nqubits, device, ising)
        append!(circ, cost_circ)
        driver_circ = driver_circuit(βs[layer], nqubits)
        append!(circ, driver_circ)
    end
    return circ
end

# ╔═╡ 0c3a53af-ff93-4a96-9cf4-375fbdde92d7
md"""
We then *train* the variational parameters. For a QAOA circuit with `n` layers, we have `2n` variational parameters: `n` β for the drivers and `n` γ for the costs. We'll use `NLOpt.jl` to perform the optimization of these parameters. This `train` function can be used for any Ising Hamiltonian that describes a QUBO problem -- we just need to generate an appropriate Hamiltonian.

In this case we will run on the `LocalSimulator`, which is free to use. The number of qubits we'll need to solve the problem is `nv * k`, where `nv` is the number of vertices in the graph and `k` is the number of colors. The `LocalSimulator` runs locally on hardware you control (e.g. your laptop, a notebook instance) and as such will only be efficient up to about 18 qubits or so on most personal computers. For this reason, we'll look at graphs with 6 vertices and try to color them with 3 colors. If you want to try larger graphs and/or more colors, some options are:

- Try with only 2 colors and use more qubits
- Run the training for longer -- more qubits will take more time & memory
- Switch from the local simulator to one of Braket's [on-demand simulators](https://docs.aws.amazon.com/braket/latest/developerguide/braket-devices.html)
- Run the local simulator on a computer with more RAM
"""

# ╔═╡ 0794c4fe-4ba6-4f8f-9cdc-fadd1a4d19a1
md"""
## Building a training function

In this section, we'll build a function to train the variational parameters $\beta$ and $\gamma$ for each layer of the QAOA circuit using `NLopt.jl` and the input Ising Hamiltonian we generate. Our training function will be *agnostic* of the specific Hamiltonian, so we can worry about generating one appropriate to our specific problem later on.

To write the training function, we'll need:
- a function `f` which takes arguments `params` and `grad` (`params` will be variational parameters $\beta$ and $\gamma$)
- a way to initialize our parameters
- a way to compute losses at each training step
- a way to track the losses and current best solution at each training iteration
"""

# ╔═╡ 3a776e14-85cc-49e7-8413-eb3891894aff
# initializing the optimization
function init_opt(qaoa_depth::Int, n_qubits::Int)
    global_min   = 1e8
    global_state = zeros(n_qubits)
    # initialize with random parameters
    βs           = rand(qaoa_depth)
    γs           = rand(qaoa_depth)
    params       = vcat(βs, γs)
    # tracker for optimization process
    tracker      = Dict("costs"=>[],
                        "params"=>[],
                        "optimal_energy"=>global_min, # best energy so far
                        "optimal_state"=>global_state, # best configuration so far
                        "opt_energies"=>Float64[], # track lowest eigenvalue at every iteration
                        "opt_states"=>[] # track configuration associated with lowest eigenvalue at every iteration
                        )
    return params, tracker
end

# ╔═╡ 49742693-693e-4957-ba26-c6e5774e6611
# generating the loss & finding the best configuration at each iteration
function apply_qaoa(qaoa_circ::Circuit, ising::SparseMatrixCSC, device::Device, nshots::Int)
    task = device(qaoa_circ, shots=nshots)
    res  = result(task)

    # result comes back as 0s and 1s
    meas_ising = Matrix{Int}(undef, nshots, size(ising, 1))
	@views for shot in 1:nshots
		meas_ising[shot, :] = res.measurements[shot][:]
	end
    # but Ising hamiltonian expects values of -1 or 1
    meas_ising = map( x -> x == 0 ? -1 : 1, meas_ising)

    # generate the energies
    J   = triu(ising, 1)
    xQx = diag(meas_ising * J * transpose(meas_ising)) # computes <x | Q | x> for every result configuration
    all_energies = xQx
    loss = sum(all_energies) / nshots

    # select the best energy
    energy_min    = minimum(all_energies)
    # find the configuration that had that energy
    min_index     = findfirst(x->x==energy_min, all_energies)
    optimal_state = meas_ising[min_index, :]

    return loss, energy_min, optimal_state
end

# ╔═╡ 0308eba7-ba4a-41f0-88f0-07fb89d99330
function train(nqubits::Int, ising::SparseMatrixCSC, device::Device; depth::Int=3, iterations::Int=100, nshots::Int=1000)
    params, tracker = init_opt(depth, nqubits)
    function f(params, grad)
        # create the circuit
        qaoa_circ = qaoa_circuit(params, device, nqubits, ising)
        loss, energy_min, optimal_state = apply_qaoa(qaoa_circ, ising, device, nshots)
        # update the tracker
        push!(tracker["opt_energies"], energy_min)
        push!(tracker["opt_states"],    optimal_state)
        if energy_min < tracker["optimal_energy"]
            tracker["optimal_energy"] = energy_min
            tracker["optimal_state"]  = optimal_state
        end
        tracker["count"] = tracker["count"] + 1
        push!(tracker["params"], params)
        push!(tracker["costs"], loss)
        return loss
    end

    opt = Opt(:LN_COBYLA, length(params))
    min_objective!(opt, f)
    maxeval!(opt, iterations)
    for i in 1:iterations
		@info "Iteration $i"
        loss, params, info = optimize(opt, params)
    end
    return tracker
end

# ╔═╡ 8f65c483-9044-4f63-ba3a-22e03a3c35eb
md"""
## Constructing the QUBO Hamiltonian

Now that we have a generic training function we can use to perform QAOA in Julia, we can turn our attention to generating the specific Ising Hamiltonian we'll need to encode the graph coloring problem we would like to solve. Again, our training function is *not unique* and could be deployed with any QUBO problem.

We can use `Graphs.jl` to generate a random graph and try to color it. We'll start off with 6 vertices and 3 colors so that the problem is tractable using the `LocalSimulator`:
"""

# ╔═╡ b4435686-5dd2-433e-ae5d-bc3cba44a4f5
nv = 6 #number of vertices

# ╔═╡ 690fe577-2bbe-4ccb-bdcd-b0c9df268b83
ne = 7 #number of edges

# ╔═╡ 28381a00-9a9d-4261-b91f-641c26416374
G  = erdos_renyi(nv, ne)

# ╔═╡ 33866077-cbec-45f6-a88e-5c787ede69cb
k  = 3 #number of colors

# ╔═╡ 647c5d92-70ff-4dad-a7ca-bff2c4ca6e68
gplot(G)

# ╔═╡ 2afde0f1-3e70-4748-9288-72e9fdfe618b
md"""
Now we need to generate an Ising Hamiltonian to represent the constraints in the problem. In order to color the graph with `k` colors, each vertex should have a different color from all its neighbors. Thus we will encode the problem in `nv * k` variables `x_{v,q}` - each one encodes whether the vertex `v` has color `q`.

For example, if we have 4 colors and two vertices, then `{x_1, x_2, x_3, x_4, x_5, x_6, x_7, x_8} = {1, 0, 0, 0, 0, 0, 1, 0}` would mean that we had colored the first vertex with color 1 and the second with color 3. The first four variables encode the presence of each of the 4 colors on the first vertex.

We'll determine the final values of these variables by representing them as qubits and performing the QAOA algorithm. We have two sets of constraints to combine:
1. Each vertex must have one color - ($x_{v,1} + x_{v,2} + \ldots + x_{v,k}$ = 1)
2. Adjacent vertices must have different colors ($x_{v,q} + x_{w,q} \leq 1$ iff $v, w$ are adjacent)

We'll encode the constraints into an `nv * k` by `nv * k` matrix that will serve as the Hamiltonian for QAOA. By finding the lowest lying eigenstate of this Hamiltonian using the training function we already wrote, we will solve the problem.

Where does the "quadratic" part arise? Following [this QUBO guide](http://leeds-faculty.colorado.edu/glover/511%20-%20QUBO%20Tutorial%20-%20updated%20version%20-%20May%204,%202019.pdf), we want to solve:

```math
\begin{align*}
\mathrm{min} y =& x^T C x \\
Ax =& b \\
\end{align*}
```

given that $x$ is equal to either $0$ or $1$. Then we can notice that:
```math
\begin{align*}
(Ax - b) =& 0 \\
(Ax - b)^T (Ax - b) =& 0 \\
\end{align*}
```

Thus, if $x$ can only take values $0$ or $1$, we can make the problem quadratic by computing this product to generate:
```math
\begin{align*}
\mathrm{min} y =& x^T C x + P_1 (Ax - b)^T (Ax - b) \\
=& x^T C x + x^T B x + c \\
=& x^T Q x + c \end{align*}
```

This will represent the optimization problem before we account for the constraints from the adjacency rules. We pick a "penalty" for each constraint ($P1$ and $P2$). These penalties are arbitrary. In our case, we need to compute:
$P_1 (x - 1)^T(x - 1)$
"""

# ╔═╡ 7bff0a07-01a3-4fae-8a47-7e8b1abd227b
# enforce that each vertex has only one color
P1 = 8

# ╔═╡ 51f4c337-5028-4f4e-b5c4-2b0d72511705
begin
	# add the P1 (x_{v,1} + ... + x_{v, k} - 1)^2 constraint
	J  = sparse(Diagonal(fill(P1,nv*k)))
	for v in 1:nv # for each vertex
	    for i in 1:k 
	        for j in i+1:k
	            J[k*(v-1) + i, k*(v-1) + j] = P1
	            J[k*(v-1) + j, k*(v-1) + i] = P1
	        end
	    end
	end
end

# ╔═╡ 9c67e81e-ba70-444c-84db-7f0bf3390c03
P2 = 8

# ╔═╡ 85a8ca28-45ef-4107-8c02-d3d04e2b7d7c
adj_mat = adjacency_matrix(G)

# ╔═╡ 5dfb5219-8e52-474a-9cad-08759092ecd2
begin
	iis, jjs, vs = findnz(triu(adj_mat))
	for (i,j) in zip(iis, jjs)
	    for color in 1:k
	        J[k*(i-1) + color, k*(j-1) + color] = P2/2
	        J[k*(j-1) + color, k*(i-1) + color] = P2/2
	    end
	end
end

# ╔═╡ d8464a8d-93d0-4919-8065-e184f6844806
md"""
## Running QAOA

Now we will run the QAOA algorithm on this graph and attempt to color it. At the end of the optimization we'll print out a trace of the best energy found at each step and a numerical representation of the solution:
"""

# ╔═╡ cb5203c6-18dd-4f4d-831f-367d46780ab5
DEPTH   = 2

# ╔═╡ 02aea1f5-67f9-4486-ad10-5946afce4fb9
SHOTS   = 1000

# ╔═╡ 27f74c2b-8f88-444c-a270-4e1c30cc9b70
device = LocalSimulator()

# ╔═╡ a20ffdbb-f006-408e-a8a1-fc4dc1c4c7b5
tracker = train(nv*k, J, device; depth=DEPTH, nshots=SHOTS)

# ╔═╡ 1c871074-ba8e-44f2-a3db-108b8c84fd67
md"""
Let's inspect the optimal solution found and see if it is a valid coloring:
"""

# ╔═╡ c936cf70-968c-467f-9587-899d938466ef
opt_state = tracker["optimal_state"]

# ╔═╡ e6e5cb2a-a0cd-4232-9d62-25ca0b74a6ee
nodefillc = distinguishable_colors(k, colorant"green")

# ╔═╡ d0d2bbc8-df71-4824-b86e-f0cf6b537278
# color of each node based on encoding in opt_state
color_inds = [Int(findfirst(x->x>0, opt_state[(k*(v-1) + 1):k*v])) for v in 1:nv]

# ╔═╡ cea94f06-91ab-4a51-8b9e-b148e37ef359
gplot(G, nodefillc=nodefillc[color_inds])

# ╔═╡ ca80e6c1-ec90-40ab-8422-ba056ca499eb
md"""
## Summing up

What we covered in this section:
- Many important optimization problems can be formulated in a quantum unconstrained binary optimization (QUBO) format
- These QUBO problems are amenable to acceleration with a hybrid classical-quantum approach.
- One such hybrid algorithm is QAOA, which we implemented above

This is a lot to take in! If it doesn't all make sense, try going through the procedure outlined above for another QUBO-type problem (max cut is a good choice, as it's a special case of graph colouring).

You might well ask, what's the point of doing all this for so few qubits? After all, extremely effective classical eigensolvers exist which could have easily found the groundstate of our Hamiltonian! For the size of problem we are considering, this is indeed true. However, we know that beyond a certain size, eigensolving is intractable even on the most advanced classical supercomputers. Our hope with algorithms like QAOA is to find techniques to make effective use of near term quantum devices (which have few qubits available), understand their performance and limitations, and prepare for a future in which we will have quantum computers with many hundreds or even thousands of qubits.
"""

# ╔═╡ 2eed0d35-7904-448c-bfe3-ab54c3207fea
md"""
### Suggestions for further exploration:
- How does the number of shots or number of layers (depth) affect the results?
- Try different types of random graphs. You can find a list of available generators in the [LightGraphs documentation](https://juliagraphs.org/LightGraphs.jl/latest/).
- Try a different QUBO problem. You can find a list of canonical examples and formulae for generating Hamiltonians from them [in this tutorial](http://leeds-faculty.colorado.edu/glover/511%20-%20QUBO%20Tutorial%20-%20updated%20version%20-%20May%204,%202019.pdf). Some suggestions are:
  - Minimum vertex cover
  - Quadratic knapsack problem
  - Max clique (you can find a formulation of this problem as QUBO [here](https://arxiv.org/pdf/1801.08649.pdf))
- Check out the [Amazon Braket QAOA tutorial](https://github.com/aws/amazon-braket-examples/blob/main/examples/hybrid_quantum_algorithms/QAOA/QAOA_braket.ipynb) for more examples
- There are graph colouring encodings for QAOA which use fewer than `nv * k` qubits -- see [this paper](https://arxiv.org/pdf/2009.07314.pdf) for an example. Can you implement their technique?
"""

# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
Braket = "19504a0f-b47d-4348-9127-acc6cc69ef67"
Cairo = "159f3aea-2a34-519c-b102-8c37f9878175"
Colors = "5ae59095-9a9b-59fe-a467-6f913c188581"
Compose = "a81c6b42-2e10-5240-aca2-a61377ecd94b"
GraphPlot = "a2cc645c-3eea-5389-862e-a155d0052231"
Graphs = "86223c79-3864-5bf0-83f7-82e725a168b6"
LinearAlgebra = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
NLopt = "76087f3c-5699-56af-9a33-bf431cd00edd"
PyBraket = "e85266a6-1825-490b-a80e-9b9469c53660"
SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"

[compat]
Braket = "~0.3.0"
Cairo = "~1.0.5"
Colors = "~0.12.10"
Compose = "~0.9.4"
GraphPlot = "~0.5.2"
Graphs = "~1.7.4"
NLopt = "~0.6.5"
PyBraket = "~0.3.0"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

julia_version = "1.9.0-beta3"
manifest_format = "2.0"
project_hash = "7c9c338d2c24d130950224875bdb0aace0f081b0"

[[deps.AWS]]
deps = ["Base64", "Compat", "Dates", "Downloads", "GitHub", "HTTP", "IniFile", "JSON", "MbedTLS", "Mocking", "OrderedCollections", "Random", "SHA", "Sockets", "URIs", "UUIDs", "XMLDict"]
git-tree-sha1 = "487d6835da9876e0362a83aec169e390872eba64"
uuid = "fbe9abb3-538b-5e4e-ba9e-bc94f4f92ebc"
version = "1.81.0"

[[deps.AWSS3]]
deps = ["AWS", "ArrowTypes", "Base64", "Compat", "Dates", "EzXML", "FilePathsBase", "MbedTLS", "Mocking", "OrderedCollections", "Retry", "SymDict", "URIs", "UUIDs", "XMLDict"]
git-tree-sha1 = "04620168e20f9c922b738fc6b7d6cfb92973ebfb"
uuid = "1c724243-ef5b-51ab-93f4-b0a88ac62a95"
version = "0.10.2"

[[deps.ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.1"

[[deps.ArnoldiMethod]]
deps = ["LinearAlgebra", "Random", "StaticArrays"]
git-tree-sha1 = "62e51b39331de8911e4a7ff6f5aaf38a5f4cc0ae"
uuid = "ec485272-7323-5ecc-a04f-4719b315124d"
version = "0.2.0"

[[deps.ArrowTypes]]
deps = ["UUIDs"]
git-tree-sha1 = "a0633b6d6efabf3f76dacd6eb1b3ec6c42ab0552"
uuid = "31f734f8-188a-4ce0-8406-c8a06bd891cd"
version = "1.2.1"

[[deps.Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"

[[deps.AxisArrays]]
deps = ["Dates", "IntervalSets", "IterTools", "RangeArrays"]
git-tree-sha1 = "1dd4d9f5beebac0c03446918741b1a03dc5e5788"
uuid = "39de3d68-74b9-583c-8d2d-e117c070f3a9"
version = "0.4.6"

[[deps.Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"

[[deps.BenchmarkTools]]
deps = ["JSON", "Logging", "Printf", "Profile", "Statistics", "UUIDs"]
git-tree-sha1 = "d9a9701b899b30332bbcb3e1679c41cce81fb0e8"
uuid = "6e4b80f9-dd63-53aa-95a3-0cdb28fa8baf"
version = "1.3.2"

[[deps.BitFlags]]
git-tree-sha1 = "43b1a4a8f797c1cddadf60499a8a077d4af2cd2d"
uuid = "d1d4a3ce-64b1-5f1a-9ba4-7e7e69966f35"
version = "0.1.7"

[[deps.Braket]]
deps = ["AWS", "AWSS3", "AxisArrays", "CSV", "Compat", "DataStructures", "Dates", "DecFP", "Distributed", "Downloads", "Graphs", "HTTP", "InteractiveUtils", "JSON3", "LinearAlgebra", "Logging", "Mocking", "NamedTupleTools", "OrderedCollections", "Random", "Statistics", "StructTypes", "Tar", "UUIDs"]
git-tree-sha1 = "f116c78dbaab8141c121a41962e28f442908051c"
uuid = "19504a0f-b47d-4348-9127-acc6cc69ef67"
version = "0.3.0"

[[deps.Bzip2_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "19a35467a82e236ff51bc17a3a44b69ef35185a2"
uuid = "6e34b625-4abd-537c-b88f-471c36dfa7a0"
version = "1.0.8+0"

[[deps.CSV]]
deps = ["CodecZlib", "Dates", "FilePathsBase", "InlineStrings", "Mmap", "Parsers", "PooledArrays", "SentinelArrays", "SnoopPrecompile", "Tables", "Unicode", "WeakRefStrings", "WorkerUtilities"]
git-tree-sha1 = "c700cce799b51c9045473de751e9319bdd1c6e94"
uuid = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
version = "0.10.9"

[[deps.Cairo]]
deps = ["Cairo_jll", "Colors", "Glib_jll", "Graphics", "Libdl", "Pango_jll"]
git-tree-sha1 = "d0b3f8b4ad16cb0a2988c6788646a5e6a17b6b1b"
uuid = "159f3aea-2a34-519c-b102-8c37f9878175"
version = "1.0.5"

[[deps.Cairo_jll]]
deps = ["Artifacts", "Bzip2_jll", "CompilerSupportLibraries_jll", "Fontconfig_jll", "FreeType2_jll", "Glib_jll", "JLLWrappers", "LZO_jll", "Libdl", "Pixman_jll", "Pkg", "Xorg_libXext_jll", "Xorg_libXrender_jll", "Zlib_jll", "libpng_jll"]
git-tree-sha1 = "4b859a208b2397a7a623a03449e4636bdb17bcf2"
uuid = "83423d85-b0ee-5818-9007-b63ccbeb887a"
version = "1.16.1+1"

[[deps.ChainRulesCore]]
deps = ["Compat", "LinearAlgebra", "SparseArrays"]
git-tree-sha1 = "c6d890a52d2c4d55d326439580c3b8d0875a77d9"
uuid = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
version = "1.15.7"

[[deps.CodecBzip2]]
deps = ["Bzip2_jll", "Libdl", "TranscodingStreams"]
git-tree-sha1 = "2e62a725210ce3c3c2e1a3080190e7ca491f18d7"
uuid = "523fee87-0ab8-5b00-afb7-3ecf72e48cfd"
version = "0.7.2"

[[deps.CodecZlib]]
deps = ["TranscodingStreams", "Zlib_jll"]
git-tree-sha1 = "9c209fb7536406834aa938fb149964b985de6c83"
uuid = "944b1d66-785c-5afd-91f1-9de20f533193"
version = "0.7.1"

[[deps.ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "eb7f0f8307f71fac7c606984ea5fb2817275d6e4"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.4"

[[deps.Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "fc08e5930ee9a4e03f84bfb5211cb54e7769758a"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.10"

[[deps.CommonSubexpressions]]
deps = ["MacroTools", "Test"]
git-tree-sha1 = "7b8a93dba8af7e3b42fecabf646260105ac373f7"
uuid = "bbf7d656-a473-5ed7-a52c-81e309532950"
version = "0.3.0"

[[deps.Compat]]
deps = ["Dates", "LinearAlgebra", "UUIDs"]
git-tree-sha1 = "00a2cccc7f098ff3b66806862d275ca3db9e6e5a"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "4.5.0"

[[deps.CompilerSupportLibraries_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"
version = "1.0.2+0"

[[deps.Compose]]
deps = ["Base64", "Colors", "DataStructures", "Dates", "IterTools", "JSON", "LinearAlgebra", "Measures", "Printf", "Random", "Requires", "Statistics", "UUIDs"]
git-tree-sha1 = "d853e57661ba3a57abcdaa201f4c9917a93487a2"
uuid = "a81c6b42-2e10-5240-aca2-a61377ecd94b"
version = "0.9.4"

[[deps.CondaPkg]]
deps = ["JSON3", "Markdown", "MicroMamba", "Pidfile", "Pkg", "TOML"]
git-tree-sha1 = "64dd885fa25c61fdf6b27e90d6adedf564ae363a"
uuid = "992eb4ea-22a4-4c89-a5bb-47a3300528ab"
version = "0.2.15"

[[deps.DataAPI]]
git-tree-sha1 = "e8119c1a33d267e16108be441a287a6981ba1630"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.14.0"

[[deps.DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "d1fff3a548102f48987a52a2e0d114fa97d730f0"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.13"

[[deps.DataValueInterfaces]]
git-tree-sha1 = "bfc1187b79289637fa0ef6d4436ebdfe6905cbd6"
uuid = "e2d170a0-9d28-54be-80f0-106bbe20a464"
version = "1.0.0"

[[deps.Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"

[[deps.DecFP]]
deps = ["DecFP_jll", "Printf", "Random", "SpecialFunctions"]
git-tree-sha1 = "a8269e0a6af8c9d9ae95d15dcfa5628285980cbb"
uuid = "55939f99-70c6-5e9b-8bb0-5071ed7d61fd"
version = "1.3.1"

[[deps.DecFP_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "e9a8da19f847bbfed4076071f6fef8665a30d9e5"
uuid = "47200ebd-12ce-5be5-abb7-8e082af23329"
version = "2.0.3+1"

[[deps.DelimitedFiles]]
deps = ["Mmap"]
git-tree-sha1 = "9e2f36d3c96a820c678f2f1f1782582fcf685bae"
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"
version = "1.9.1"

[[deps.DiffResults]]
deps = ["StaticArraysCore"]
git-tree-sha1 = "782dd5f4561f5d267313f23853baaaa4c52ea621"
uuid = "163ba53b-c6d8-5494-b064-1a9d43ac40c5"
version = "1.1.0"

[[deps.DiffRules]]
deps = ["IrrationalConstants", "LogExpFunctions", "NaNMath", "Random", "SpecialFunctions"]
git-tree-sha1 = "c5b6685d53f933c11404a3ae9822afe30d522494"
uuid = "b552c78f-8df3-52c6-915a-8e097449b14b"
version = "1.12.2"

[[deps.Distributed]]
deps = ["Random", "Serialization", "Sockets"]
uuid = "8ba89e20-285c-5b6f-9357-94700520ee1b"

[[deps.DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "2fb1e02f2b635d0845df5d7c167fec4dd739b00d"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.9.3"

[[deps.Downloads]]
deps = ["ArgTools", "FileWatching", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.6.0"

[[deps.Expat_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "bad72f730e9e91c08d9427d5e8db95478a3c323d"
uuid = "2e619515-83b5-522b-bb60-26c02a35a201"
version = "2.4.8+0"

[[deps.ExprTools]]
git-tree-sha1 = "56559bbef6ca5ea0c0818fa5c90320398a6fbf8d"
uuid = "e2ba6199-217a-4e67-a87a-7c52f15ade04"
version = "0.1.8"

[[deps.EzXML]]
deps = ["Printf", "XML2_jll"]
git-tree-sha1 = "0fa3b52a04a4e210aeb1626def9c90df3ae65268"
uuid = "8f5d6c58-4d21-5cfd-889c-e3ad7ee6a615"
version = "1.1.0"

[[deps.FilePathsBase]]
deps = ["Compat", "Dates", "Mmap", "Printf", "Test", "UUIDs"]
git-tree-sha1 = "e27c4ebe80e8699540f2d6c805cc12203b614f12"
uuid = "48062228-2e41-5def-b9a4-89aafe57970f"
version = "0.9.20"

[[deps.FileWatching]]
uuid = "7b1f6079-737a-58dc-b8bc-7a2ca5c1b5ee"

[[deps.FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "335bfdceacc84c5cdf16aadc768aa5ddfc5383cc"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.4"

[[deps.Fontconfig_jll]]
deps = ["Artifacts", "Bzip2_jll", "Expat_jll", "FreeType2_jll", "JLLWrappers", "Libdl", "Libuuid_jll", "Pkg", "Zlib_jll"]
git-tree-sha1 = "21efd19106a55620a188615da6d3d06cd7f6ee03"
uuid = "a3f928ae-7b40-5064-980b-68af3947d34b"
version = "2.13.93+0"

[[deps.ForwardDiff]]
deps = ["CommonSubexpressions", "DiffResults", "DiffRules", "LinearAlgebra", "LogExpFunctions", "NaNMath", "Preferences", "Printf", "Random", "SpecialFunctions", "StaticArrays"]
git-tree-sha1 = "a69dd6db8a809f78846ff259298678f0d6212180"
uuid = "f6369f11-7733-5829-9624-2563aa707210"
version = "0.10.34"

[[deps.FreeType2_jll]]
deps = ["Artifacts", "Bzip2_jll", "JLLWrappers", "Libdl", "Pkg", "Zlib_jll"]
git-tree-sha1 = "87eb71354d8ec1a96d4a7636bd57a7347dde3ef9"
uuid = "d7e528f0-a631-5988-bf34-fe36492bcfd7"
version = "2.10.4+0"

[[deps.FriBidi_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "aa31987c2ba8704e23c6c8ba8a4f769d5d7e4f91"
uuid = "559328eb-81f9-559d-9380-de523a88c83c"
version = "1.0.10+0"

[[deps.Future]]
deps = ["Random"]
uuid = "9fa8497b-333b-5362-9e8d-4d0656e87820"

[[deps.Gettext_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "JLLWrappers", "Libdl", "Libiconv_jll", "Pkg", "XML2_jll"]
git-tree-sha1 = "9b02998aba7bf074d14de89f9d37ca24a1a0b046"
uuid = "78b55507-aeef-58d4-861c-77aaff3498b1"
version = "0.21.0+0"

[[deps.GitHub]]
deps = ["Base64", "Dates", "HTTP", "JSON", "MbedTLS", "Sockets", "SodiumSeal", "URIs"]
git-tree-sha1 = "5688002de970b9eee14b7af7bbbd1fdac10c9bbe"
uuid = "bc5e4493-9b4d-5f90-b8aa-2b2bcaad7a26"
version = "5.8.2"

[[deps.Glib_jll]]
deps = ["Artifacts", "Gettext_jll", "JLLWrappers", "Libdl", "Libffi_jll", "Libiconv_jll", "Libmount_jll", "PCRE2_jll", "Pkg", "Zlib_jll"]
git-tree-sha1 = "d3b3624125c1474292d0d8ed0f65554ac37ddb23"
uuid = "7746bdde-850d-59dc-9ae8-88ece973131d"
version = "2.74.0+2"

[[deps.GraphPlot]]
deps = ["ArnoldiMethod", "ColorTypes", "Colors", "Compose", "DelimitedFiles", "Graphs", "LinearAlgebra", "Random", "SparseArrays"]
git-tree-sha1 = "5cd479730a0cb01f880eff119e9803c13f214cab"
uuid = "a2cc645c-3eea-5389-862e-a155d0052231"
version = "0.5.2"

[[deps.Graphics]]
deps = ["Colors", "LinearAlgebra", "NaNMath"]
git-tree-sha1 = "d61890399bc535850c4bf08e4e0d3a7ad0f21cbd"
uuid = "a2bd30eb-e257-5431-a919-1863eab51364"
version = "1.1.2"

[[deps.Graphite2_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "344bf40dcab1073aca04aa0df4fb092f920e4011"
uuid = "3b182d85-2403-5c21-9c21-1e1f0cc25472"
version = "1.3.14+0"

[[deps.Graphs]]
deps = ["ArnoldiMethod", "Compat", "DataStructures", "Distributed", "Inflate", "LinearAlgebra", "Random", "SharedArrays", "SimpleTraits", "SparseArrays", "Statistics"]
git-tree-sha1 = "ba2d094a88b6b287bd25cfa86f301e7693ffae2f"
uuid = "86223c79-3864-5bf0-83f7-82e725a168b6"
version = "1.7.4"

[[deps.HTTP]]
deps = ["Base64", "CodecZlib", "Dates", "IniFile", "Logging", "LoggingExtras", "MbedTLS", "NetworkOptions", "OpenSSL", "Random", "SimpleBufferStream", "Sockets", "URIs", "UUIDs"]
git-tree-sha1 = "eb5aa5e3b500e191763d35198f859e4b40fff4a6"
uuid = "cd3eb016-35fb-5094-929b-558a96fad6f3"
version = "1.7.3"

[[deps.HarfBuzz_jll]]
deps = ["Artifacts", "Cairo_jll", "Fontconfig_jll", "FreeType2_jll", "Glib_jll", "Graphite2_jll", "JLLWrappers", "Libdl", "Libffi_jll", "Pkg"]
git-tree-sha1 = "129acf094d168394e80ee1dc4bc06ec835e510a3"
uuid = "2e76f6c2-a576-52d4-95c1-20adfe4de566"
version = "2.8.1+1"

[[deps.Inflate]]
git-tree-sha1 = "5cd07aab533df5170988219191dfad0519391428"
uuid = "d25df0c9-e2be-5dd7-82c8-3ad0b3e990b9"
version = "0.1.3"

[[deps.IniFile]]
git-tree-sha1 = "f550e6e32074c939295eb5ea6de31849ac2c9625"
uuid = "83e8ac13-25f8-5344-8a64-a9f2b223428f"
version = "0.5.1"

[[deps.InlineStrings]]
deps = ["Parsers"]
git-tree-sha1 = "9cc2baf75c6d09f9da536ddf58eb2f29dedaf461"
uuid = "842dd82b-1e85-43dc-bf29-5d0ee9dffc48"
version = "1.4.0"

[[deps.InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"

[[deps.IntervalSets]]
deps = ["Dates", "Random", "Statistics"]
git-tree-sha1 = "16c0cc91853084cb5f58a78bd209513900206ce6"
uuid = "8197267c-284f-5f27-9208-e0e47529a953"
version = "0.7.4"

[[deps.IrrationalConstants]]
git-tree-sha1 = "7fd44fd4ff43fc60815f8e764c0f352b83c49151"
uuid = "92d709cd-6900-40b7-9082-c6be49f344b6"
version = "0.1.1"

[[deps.IterTools]]
git-tree-sha1 = "fa6287a4469f5e048d763df38279ee729fbd44e5"
uuid = "c8e1da08-722c-5040-9ed9-7db0dc04731e"
version = "1.4.0"

[[deps.IteratorInterfaceExtensions]]
git-tree-sha1 = "a3f24677c21f5bbe9d2a714f95dcd58337fb2856"
uuid = "82899510-4779-5014-852e-03e436cf321d"
version = "1.0.0"

[[deps.JLLWrappers]]
deps = ["Preferences"]
git-tree-sha1 = "abc9885a7ca2052a736a600f7fa66209f96506e1"
uuid = "692b3bcd-3c85-4b1f-b108-f13ce0eb3210"
version = "1.4.1"

[[deps.JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "3c837543ddb02250ef42f4738347454f95079d4e"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.3"

[[deps.JSON3]]
deps = ["Dates", "Mmap", "Parsers", "SnoopPrecompile", "StructTypes", "UUIDs"]
git-tree-sha1 = "84b10656a41ef564c39d2d477d7236966d2b5683"
uuid = "0f8b85d8-7281-11e9-16c2-39a750bddbf1"
version = "1.12.0"

[[deps.LZO_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "e5b909bcf985c5e2605737d2ce278ed791b89be6"
uuid = "dd4b983a-f0e5-5f8d-a1b7-129d4a5fb1ac"
version = "2.10.1+0"

[[deps.LazyArtifacts]]
deps = ["Artifacts", "Pkg"]
uuid = "4af54fe1-eca0-43a8-85a7-787d91b784e3"

[[deps.LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.3"

[[deps.LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "7.84.0+0"

[[deps.LibGit2]]
deps = ["Base64", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"

[[deps.LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.10.2+0"

[[deps.Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"

[[deps.Libffi_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "0b4a5d71f3e5200a7dff793393e09dfc2d874290"
uuid = "e9f186c6-92d2-5b65-8a66-fee21dc1b490"
version = "3.2.2+1"

[[deps.Libgcrypt_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Libgpg_error_jll", "Pkg"]
git-tree-sha1 = "64613c82a59c120435c067c2b809fc61cf5166ae"
uuid = "d4300ac3-e22c-5743-9152-c294e39db1e4"
version = "1.8.7+0"

[[deps.Libgpg_error_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "c333716e46366857753e273ce6a69ee0945a6db9"
uuid = "7add5ba3-2f88-524e-9cd5-f83b8a55f7b8"
version = "1.42.0+0"

[[deps.Libiconv_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "c7cb1f5d892775ba13767a87c7ada0b980ea0a71"
uuid = "94ce4f54-9a6c-5748-9c1c-f9c7231a4531"
version = "1.16.1+2"

[[deps.Libmount_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "9c30530bf0effd46e15e0fdcf2b8636e78cbbd73"
uuid = "4b2f31a3-9ecc-558c-b454-b3730dcb73e9"
version = "2.35.0+0"

[[deps.Libuuid_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "7f3efec06033682db852f8b3bc3c1d2b0a0ab066"
uuid = "38a345b3-de98-5d2b-a5d3-14cd9215e700"
version = "2.36.0+0"

[[deps.LinearAlgebra]]
deps = ["Libdl", "OpenBLAS_jll", "libblastrampoline_jll"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"

[[deps.LogExpFunctions]]
deps = ["DocStringExtensions", "IrrationalConstants", "LinearAlgebra"]
git-tree-sha1 = "45b288af6956e67e621c5cbb2d75a261ab58300b"
uuid = "2ab3a3ac-af41-5b50-aa03-7779005ae688"
version = "0.3.20"

    [deps.LogExpFunctions.extensions]
    ChainRulesCoreExt = "ChainRulesCore"
    ChangesOfVariablesExt = "ChangesOfVariables"
    InverseFunctionsExt = "InverseFunctions"

    [deps.LogExpFunctions.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    ChangesOfVariables = "9e997f8a-9a97-42d5-a9f1-ce6bfc15e2c0"
    InverseFunctions = "3587e190-3f89-42d0-90ee-14403ec27112"

[[deps.Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"

[[deps.LoggingExtras]]
deps = ["Dates", "Logging"]
git-tree-sha1 = "cedb76b37bc5a6c702ade66be44f831fa23c681e"
uuid = "e6f89c97-d47a-5376-807f-9c37f3926c36"
version = "1.0.0"

[[deps.MacroTools]]
deps = ["Markdown", "Random"]
git-tree-sha1 = "42324d08725e200c23d4dfb549e0d5d89dede2d2"
uuid = "1914dd2f-81c6-5fcd-8719-6d5c9610ff09"
version = "0.5.10"

[[deps.Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"

[[deps.MathOptInterface]]
deps = ["BenchmarkTools", "CodecBzip2", "CodecZlib", "DataStructures", "ForwardDiff", "JSON", "LinearAlgebra", "MutableArithmetics", "NaNMath", "OrderedCollections", "Printf", "SnoopPrecompile", "SparseArrays", "SpecialFunctions", "Test", "Unicode"]
git-tree-sha1 = "b577d6c6b484f35fc27c1e767dc32458815da0e5"
uuid = "b8f27783-ece8-5eb3-8dc8-9495eed66fee"
version = "1.11.5"

[[deps.MathProgBase]]
deps = ["LinearAlgebra", "SparseArrays"]
git-tree-sha1 = "9abbe463a1e9fc507f12a69e7f29346c2cdc472c"
uuid = "fdba3010-5040-5b88-9595-932c9decdf73"
version = "0.7.8"

[[deps.MbedTLS]]
deps = ["Dates", "MbedTLS_jll", "MozillaCACerts_jll", "Random", "Sockets"]
git-tree-sha1 = "03a9b9718f5682ecb107ac9f7308991db4ce395b"
uuid = "739be429-bea8-5141-9913-cc70e7f3736d"
version = "1.1.7"

[[deps.MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.28.0+0"

[[deps.Measures]]
git-tree-sha1 = "c13304c81eec1ed3af7fc20e75fb6b26092a1102"
uuid = "442fdcdd-2543-5da2-b0f3-8c86c306513e"
version = "0.3.2"

[[deps.MicroMamba]]
deps = ["Pkg", "Scratch", "micromamba_jll"]
git-tree-sha1 = "a6a4771aba1dc8942bc0f44ff9f8ee0f893ef888"
uuid = "0b3b1443-0f03-428d-bdfb-f27f9c1191ea"
version = "0.1.12"

[[deps.Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"

[[deps.Mocking]]
deps = ["Compat", "ExprTools"]
git-tree-sha1 = "c272302b22479a24d1cf48c114ad702933414f80"
uuid = "78c3b35d-d492-501b-9361-3d52fe80e533"
version = "0.7.5"

[[deps.MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2022.10.11"

[[deps.MutableArithmetics]]
deps = ["LinearAlgebra", "SparseArrays", "Test"]
git-tree-sha1 = "aa532179d4a643d4bd9f328589ca01fa20a0d197"
uuid = "d8a4904e-b15c-11e9-3269-09a3773c0cb0"
version = "1.1.0"

[[deps.NLopt]]
deps = ["MathOptInterface", "MathProgBase", "NLopt_jll"]
git-tree-sha1 = "5a7e32c569200a8a03c3d55d286254b0321cd262"
uuid = "76087f3c-5699-56af-9a33-bf431cd00edd"
version = "0.6.5"

[[deps.NLopt_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "9b1f15a08f9d00cdb2761dcfa6f453f5d0d6f973"
uuid = "079eb43e-fd8e-5478-9966-2cf3e3edb778"
version = "2.7.1+0"

[[deps.NaNMath]]
deps = ["OpenLibm_jll"]
git-tree-sha1 = "a7c3d1da1189a1c2fe843a3bfa04d18d20eb3211"
uuid = "77ba4419-2d1f-58cd-9bb1-8ffee604a2e3"
version = "1.0.1"

[[deps.NamedTupleTools]]
git-tree-sha1 = "90914795fc59df44120fe3fff6742bb0d7adb1d0"
uuid = "d9ec5142-1e00-5aa0-9d6a-321866360f50"
version = "0.14.3"

[[deps.NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[deps.OpenBLAS_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Libdl"]
uuid = "4536629a-c528-5b80-bd46-f80d51c5b363"
version = "0.3.21+0"

[[deps.OpenLibm_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "05823500-19ac-5b8b-9628-191a04bc5112"
version = "0.8.1+0"

[[deps.OpenSSL]]
deps = ["BitFlags", "Dates", "MozillaCACerts_jll", "OpenSSL_jll", "Sockets"]
git-tree-sha1 = "6503b77492fd7fcb9379bf73cd31035670e3c509"
uuid = "4d8831e6-92b7-49fb-bdf8-b643e874388c"
version = "1.3.3"

[[deps.OpenSSL_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "f6e9dba33f9f2c44e08a020b0caf6903be540004"
uuid = "458c3c95-2e84-50aa-8efc-19380b2a3a95"
version = "1.1.19+0"

[[deps.OpenSpecFun_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "13652491f6856acfd2db29360e1bbcd4565d04f1"
uuid = "efe28fd5-8261-553b-a9e1-b2916fc3738e"
version = "0.5.5+0"

[[deps.OrderedCollections]]
git-tree-sha1 = "85f8e6578bf1f9ee0d11e7bb1b1456435479d47c"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.4.1"

[[deps.PCRE2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "efcefdf7-47ab-520b-bdef-62a2eaa19f15"
version = "10.42.0+0"

[[deps.Pango_jll]]
deps = ["Artifacts", "Cairo_jll", "Fontconfig_jll", "FreeType2_jll", "FriBidi_jll", "Glib_jll", "HarfBuzz_jll", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "84a314e3926ba9ec66ac097e3635e270986b0f10"
uuid = "36c8627f-9965-5494-a995-c6b170f724f3"
version = "1.50.9+0"

[[deps.Parsers]]
deps = ["Dates", "SnoopPrecompile"]
git-tree-sha1 = "8175fc2b118a3755113c8e68084dc1a9e63c61ee"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "2.5.3"

[[deps.Pidfile]]
deps = ["FileWatching", "Test"]
git-tree-sha1 = "2d8aaf8ee10df53d0dfb9b8ee44ae7c04ced2b03"
uuid = "fa939f87-e72e-5be4-a000-7fc836dbe307"
version = "1.3.0"

[[deps.Pixman_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "b4f5d02549a10e20780a24fce72bea96b6329e29"
uuid = "30392449-352a-5448-841d-b1acce4e97dc"
version = "0.40.1+0"

[[deps.Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "FileWatching", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "REPL", "Random", "SHA", "Serialization", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
version = "1.9.0"

[[deps.PooledArrays]]
deps = ["DataAPI", "Future"]
git-tree-sha1 = "a6062fe4063cdafe78f4a0a81cfffb89721b30e7"
uuid = "2dfb63ee-cc39-5dd5-95bd-886bf059d720"
version = "1.4.2"

[[deps.Preferences]]
deps = ["TOML"]
git-tree-sha1 = "47e5f437cc0e7ef2ce8406ce1e7e24d44915f88d"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.3.0"

[[deps.Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"

[[deps.Profile]]
deps = ["Printf"]
uuid = "9abbd945-dff8-562f-b5e8-e1ebf5ef1b79"

[[deps.PyBraket]]
deps = ["Braket", "CondaPkg", "DataStructures", "LinearAlgebra", "PythonCall", "Statistics", "StructTypes"]
git-tree-sha1 = "64f9f4e72ef170541eecbb9398906114a33b9a34"
uuid = "e85266a6-1825-490b-a80e-9b9469c53660"
version = "0.3.0"

[[deps.PythonCall]]
deps = ["CondaPkg", "Dates", "Libdl", "MacroTools", "Markdown", "Pkg", "REPL", "Requires", "Serialization", "Tables", "UnsafePointers"]
git-tree-sha1 = "1052188e0a017d4f4f261f12307e1fa1b5b48588"
uuid = "6099a3de-0909-46bc-b1f4-468b9a2dfc0d"
version = "0.9.10"

[[deps.REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"

[[deps.Random]]
deps = ["SHA", "Serialization"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"

[[deps.RangeArrays]]
git-tree-sha1 = "b9039e93773ddcfc828f12aadf7115b4b4d225f5"
uuid = "b3c3ace0-ae52-54e7-9d0b-2c1406fd6b9d"
version = "0.3.2"

[[deps.Reexport]]
git-tree-sha1 = "45e428421666073eab6f2da5c9d310d99bb12f9b"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.2.2"

[[deps.Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "838a3a4188e2ded87a4f9f184b4b0d78a1e91cb7"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.3.0"

[[deps.Retry]]
git-tree-sha1 = "41ac127cd281bb33e42aba46a9d3b25cd35fc6d5"
uuid = "20febd7b-183b-5ae2-ac4a-720e7ce64774"
version = "0.4.1"

[[deps.SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"
version = "0.7.0"

[[deps.Scratch]]
deps = ["Dates"]
git-tree-sha1 = "f94f779c94e58bf9ea243e77a37e16d9de9126bd"
uuid = "6c6a2e73-6563-6170-7368-637461726353"
version = "1.1.1"

[[deps.SentinelArrays]]
deps = ["Dates", "Random"]
git-tree-sha1 = "c02bd3c9c3fc8463d3591a62a378f90d2d8ab0f3"
uuid = "91c51154-3ec4-41a3-a24f-3f23e20d615c"
version = "1.3.17"

[[deps.Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"

[[deps.SharedArrays]]
deps = ["Distributed", "Mmap", "Random", "Serialization"]
uuid = "1a1011a3-84de-559e-8e89-a11a2f7dc383"

[[deps.SimpleBufferStream]]
git-tree-sha1 = "874e8867b33a00e784c8a7e4b60afe9e037b74e1"
uuid = "777ac1f9-54b0-4bf8-805c-2214025038e7"
version = "1.1.0"

[[deps.SimpleTraits]]
deps = ["InteractiveUtils", "MacroTools"]
git-tree-sha1 = "5d7e3f4e11935503d3ecaf7186eac40602e7d231"
uuid = "699a6c99-e7fa-54fc-8d76-47d257e15c1d"
version = "0.9.4"

[[deps.SnoopPrecompile]]
deps = ["Preferences"]
git-tree-sha1 = "e760a70afdcd461cf01a575947738d359234665c"
uuid = "66db9d55-30c0-4569-8b51-7e840670fc0c"
version = "1.0.3"

[[deps.Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"

[[deps.SodiumSeal]]
deps = ["Base64", "Libdl", "libsodium_jll"]
git-tree-sha1 = "80cef67d2953e33935b41c6ab0a178b9987b1c99"
uuid = "2133526b-2bfb-4018-ac12-889fb3908a75"
version = "0.1.1"

[[deps.SparseArrays]]
deps = ["Libdl", "LinearAlgebra", "Random", "Serialization", "SuiteSparse_jll"]
uuid = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"

[[deps.SpecialFunctions]]
deps = ["ChainRulesCore", "IrrationalConstants", "LogExpFunctions", "OpenLibm_jll", "OpenSpecFun_jll"]
git-tree-sha1 = "d75bda01f8c31ebb72df80a46c88b25d1c79c56d"
uuid = "276daf66-3868-5448-9aa4-cd146d93841b"
version = "2.1.7"

[[deps.StaticArrays]]
deps = ["LinearAlgebra", "Random", "StaticArraysCore", "Statistics"]
git-tree-sha1 = "6954a456979f23d05085727adb17c4551c19ecd1"
uuid = "90137ffa-7385-5640-81b9-e52037218182"
version = "1.5.12"

[[deps.StaticArraysCore]]
git-tree-sha1 = "6b7ba252635a5eff6a0b0664a41ee140a1c9e72a"
uuid = "1e83bf80-4336-4d27-bf5d-d5a4f845583c"
version = "1.4.0"

[[deps.Statistics]]
deps = ["LinearAlgebra", "SparseArrays"]
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"
version = "1.9.0"

[[deps.StructTypes]]
deps = ["Dates", "UUIDs"]
git-tree-sha1 = "ca4bccb03acf9faaf4137a9abc1881ed1841aa70"
uuid = "856f2bd8-1eba-4b0a-8007-ebc267875bd4"
version = "1.10.0"

[[deps.SuiteSparse_jll]]
deps = ["Artifacts", "Libdl", "Pkg", "libblastrampoline_jll"]
uuid = "bea87d4a-7f5b-5778-9afe-8cc45184846c"
version = "5.10.1+0"

[[deps.SymDict]]
deps = ["Test"]
git-tree-sha1 = "0108ccdaea3ef69d9680eeafc8d5ad198b896ec8"
uuid = "2da68c74-98d7-5633-99d6-8493888d7b1e"
version = "0.3.0"

[[deps.TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.3"

[[deps.TableTraits]]
deps = ["IteratorInterfaceExtensions"]
git-tree-sha1 = "c06b2f539df1c6efa794486abfb6ed2022561a39"
uuid = "3783bdb8-4a98-5b6b-af9a-565f29a5fe9c"
version = "1.0.1"

[[deps.Tables]]
deps = ["DataAPI", "DataValueInterfaces", "IteratorInterfaceExtensions", "LinearAlgebra", "OrderedCollections", "TableTraits", "Test"]
git-tree-sha1 = "c79322d36826aa2f4fd8ecfa96ddb47b174ac78d"
uuid = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"
version = "1.10.0"

[[deps.Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"
version = "1.10.0"

[[deps.Test]]
deps = ["InteractiveUtils", "Logging", "Random", "Serialization"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"

[[deps.TranscodingStreams]]
deps = ["Random", "Test"]
git-tree-sha1 = "94f38103c984f89cf77c402f2a68dbd870f8165f"
uuid = "3bb67fe8-82b1-5028-8e26-92a6c54297fa"
version = "0.9.11"

[[deps.URIs]]
git-tree-sha1 = "ac00576f90d8a259f2c9d823e91d1de3fd44d348"
uuid = "5c2747f8-b7ea-4ff2-ba2e-563bfd36b1d4"
version = "1.4.1"

[[deps.UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"

[[deps.Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"

[[deps.UnsafePointers]]
git-tree-sha1 = "c81331b3b2e60a982be57c046ec91f599ede674a"
uuid = "e17b2a0c-0bdf-430a-bd0c-3a23cae4ff39"
version = "1.0.0"

[[deps.WeakRefStrings]]
deps = ["DataAPI", "InlineStrings", "Parsers"]
git-tree-sha1 = "b1be2855ed9ed8eac54e5caff2afcdb442d52c23"
uuid = "ea10d353-3f73-51f8-a26c-33c1cb351aa5"
version = "1.4.2"

[[deps.WorkerUtilities]]
git-tree-sha1 = "cd1659ba0d57b71a464a29e64dbc67cfe83d54e7"
uuid = "76eceee3-57b5-4d4a-8e66-0e911cebbf60"
version = "1.6.1"

[[deps.XML2_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Libiconv_jll", "Pkg", "Zlib_jll"]
git-tree-sha1 = "93c41695bc1c08c46c5899f4fe06d6ead504bb73"
uuid = "02c8fc9c-b97f-50b9-bbe4-9be30ff0a78a"
version = "2.10.3+0"

[[deps.XMLDict]]
deps = ["EzXML", "IterTools", "OrderedCollections"]
git-tree-sha1 = "d9a3faf078210e477b291c79117676fca54da9dd"
uuid = "228000da-037f-5747-90a9-8195ccbf91a5"
version = "0.4.1"

[[deps.XSLT_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Libgcrypt_jll", "Libgpg_error_jll", "Libiconv_jll", "Pkg", "XML2_jll", "Zlib_jll"]
git-tree-sha1 = "91844873c4085240b95e795f692c4cec4d805f8a"
uuid = "aed1982a-8fda-507f-9586-7b0439959a61"
version = "1.1.34+0"

[[deps.Xorg_libX11_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libxcb_jll", "Xorg_xtrans_jll"]
git-tree-sha1 = "5be649d550f3f4b95308bf0183b82e2582876527"
uuid = "4f6342f7-b3d2-589e-9d20-edeb45f2b2bc"
version = "1.6.9+4"

[[deps.Xorg_libXau_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "4e490d5c960c314f33885790ed410ff3a94ce67e"
uuid = "0c0b7dd1-d40b-584c-a123-a41640f87eec"
version = "1.0.9+4"

[[deps.Xorg_libXdmcp_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "4fe47bd2247248125c428978740e18a681372dd4"
uuid = "a3789734-cfe1-5b06-b2d0-1dd0d9d62d05"
version = "1.1.3+4"

[[deps.Xorg_libXext_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libX11_jll"]
git-tree-sha1 = "b7c0aa8c376b31e4852b360222848637f481f8c3"
uuid = "1082639a-0dae-5f34-9b06-72781eeb8cb3"
version = "1.3.4+4"

[[deps.Xorg_libXrender_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libX11_jll"]
git-tree-sha1 = "19560f30fd49f4d4efbe7002a1037f8c43d43b96"
uuid = "ea2f1a96-1ddc-540d-b46f-429655e07cfa"
version = "0.9.10+4"

[[deps.Xorg_libpthread_stubs_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "6783737e45d3c59a4a4c4091f5f88cdcf0908cbb"
uuid = "14d82f49-176c-5ed1-bb49-ad3f5cbd8c74"
version = "0.1.0+3"

[[deps.Xorg_libxcb_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "XSLT_jll", "Xorg_libXau_jll", "Xorg_libXdmcp_jll", "Xorg_libpthread_stubs_jll"]
git-tree-sha1 = "daf17f441228e7a3833846cd048892861cff16d6"
uuid = "c7cfdc94-dc32-55de-ac96-5a1b8d977c5b"
version = "1.13.0+3"

[[deps.Xorg_xtrans_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "79c31e7844f6ecf779705fbc12146eb190b7d845"
uuid = "c5fb5394-a638-5e4d-96e5-b29de1b5cf10"
version = "1.4.0+3"

[[deps.Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.13+0"

[[deps.libblastrampoline_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850b90-86db-534c-a0d3-1478176c7d93"
version = "5.2.0+0"

[[deps.libpng_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Zlib_jll"]
git-tree-sha1 = "94d180a6d2b5e55e447e2d27a29ed04fe79eb30c"
uuid = "b53b4c65-9356-5827-b1ea-8c7a1a84506f"
version = "1.6.38+0"

[[deps.libsodium_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "848ab3d00fe39d6fbc2a8641048f8f272af1c51e"
uuid = "a9144af2-ca23-56d9-984f-0d03f7b5ccf8"
version = "1.0.20+0"

[[deps.micromamba_jll]]
deps = ["Artifacts", "JLLWrappers", "LazyArtifacts", "Libdl", "Pkg"]
git-tree-sha1 = "80ddb5f510c650de288ecd548ebc3de557ffb3e2"
uuid = "f8abcde7-e9b7-5caa-b8af-a437887ae8e4"
version = "1.2.0+0"

[[deps.nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.48.0+0"

[[deps.p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
version = "17.4.0+0"
"""

# ╔═╡ Cell order:
# ╟─7112d37e-9f1e-11ed-313d-11843dc0a14e
# ╟─12feb970-e7e0-4e96-b537-5afe8e82ac17
# ╟─318db32f-e76c-4a46-9557-bd93a660505a
# ╟─3023c427-e656-4468-951e-669952b3124b
# ╟─b71a0897-b1e3-4ea7-89e0-53ff4aca5e57
# ╠═5ba10044-c01a-486a-b6bd-bad198bf1960
# ╠═a1592cdc-0f2e-4794-96f7-328347b27481
# ╠═c32edfaf-5191-4428-b86f-4077222b1b06
# ╠═03970aa4-a5e1-44cf-b06f-dfabe28913d8
# ╠═fd229791-a4a5-4b59-8ca5-e694a25d698d
# ╟─0c3a53af-ff93-4a96-9cf4-375fbdde92d7
# ╟─0794c4fe-4ba6-4f8f-9cdc-fadd1a4d19a1
# ╠═3a776e14-85cc-49e7-8413-eb3891894aff
# ╠═49742693-693e-4957-ba26-c6e5774e6611
# ╠═0308eba7-ba4a-41f0-88f0-07fb89d99330
# ╟─8f65c483-9044-4f63-ba3a-22e03a3c35eb
# ╠═ec25630f-2ca7-421a-8559-3046144dd236
# ╠═b4435686-5dd2-433e-ae5d-bc3cba44a4f5
# ╠═690fe577-2bbe-4ccb-bdcd-b0c9df268b83
# ╠═28381a00-9a9d-4261-b91f-641c26416374
# ╠═33866077-cbec-45f6-a88e-5c787ede69cb
# ╠═647c5d92-70ff-4dad-a7ca-bff2c4ca6e68
# ╟─2afde0f1-3e70-4748-9288-72e9fdfe618b
# ╠═7bff0a07-01a3-4fae-8a47-7e8b1abd227b
# ╠═51f4c337-5028-4f4e-b5c4-2b0d72511705
# ╠═9c67e81e-ba70-444c-84db-7f0bf3390c03
# ╠═85a8ca28-45ef-4107-8c02-d3d04e2b7d7c
# ╠═5dfb5219-8e52-474a-9cad-08759092ecd2
# ╟─d8464a8d-93d0-4919-8065-e184f6844806
# ╠═cb5203c6-18dd-4f4d-831f-367d46780ab5
# ╠═02aea1f5-67f9-4486-ad10-5946afce4fb9
# ╠═27f74c2b-8f88-444c-a270-4e1c30cc9b70
# ╠═a20ffdbb-f006-408e-a8a1-fc4dc1c4c7b5
# ╟─1c871074-ba8e-44f2-a3db-108b8c84fd67
# ╠═c936cf70-968c-467f-9587-899d938466ef
# ╠═e6e5cb2a-a0cd-4232-9d62-25ca0b74a6ee
# ╠═d0d2bbc8-df71-4824-b86e-f0cf6b537278
# ╠═cea94f06-91ab-4a51-8b9e-b148e37ef359
# ╟─ca80e6c1-ec90-40ab-8422-ba056ca499eb
# ╟─2eed0d35-7904-448c-bfe3-ab54c3207fea
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
